----------------------------------------------------------------------------------
-- Company: Digilent RO
-- Engineer: Mircea Dabacan & Mihai Corovei
-- 
-- Create Date:    18:00:57 03/02/2008 
-- Module Name:    Hb5RefProj - Mixt (Structural and Behavioral)
-- Project Name:   Hb5RefProj
-- Description: This project demonstrates the use of the Digilent HB5 Reference 
--              Component, HB5RefComp, with a DC motor.
--              The SA and SB hall sensors signals are used, to calculate the 
--              speed of the motors.
--              The project can also be used with the Pmod HB3.
--              The above component is instantiated to control a Pmod HB5
--              The adjustBcd component provides a BCD value, adjustable with
--              btnFaster and btnSlower.
--              This value is used to set the duty factor of pinEn. The value is 
--              also shown on the 7 segment display.
--              swDir sets the direction for the motor, after being secured in
--              the HB5RefComp. The direction is also shown on the 7 segment 
--              display as "-" (for minus) or nothing (for plus).
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Hb5RefProj is
    Port ( ck : in  STD_LOGIC;         -- system clock (50MHz)
           btnFaster : in  STD_LOGIC;  -- rise Duty Factor command
           btnSlower : in  STD_LOGIC;  -- lower Duty Factor command
           swDir : in  STD_LOGIC;      -- Direction command
		   swSelDisp : in  STD_LOGIC;  -- Display selection: '0' = PWM, '1' = Position
           pinEn : out  STD_LOGIC;     -- HB5(HB3) pin En
           pinDir : out  STD_LOGIC;    -- HB5(HB3) pin Dir
           pinSA : in  STD_LOGIC;     -- HB5(HB3) pin SA
           pinSB : in  STD_LOGIC;     -- HB5(HB3) pin SB
           seg: out std_logic_vector(6 downto 0); -- segment outputs
           an : out std_logic_vector(3 downto 0));-- anode select signals
end Hb5RefProj;

architecture Mixt of Hb5RefProj is

	COMPONENT adjustBcd
	PORT(
		ck : IN std_logic;
		bitCountUp : IN std_logic;
		bitCountDown : IN std_logic;          
		vecBcdOut : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;

	COMPONENT HB5RefComp
	PORT(
		ck : IN std_logic;
		bitDirIn : IN std_logic;
		vecDfIn : IN std_logic_vector(7 downto 0);          
		bitEnOut : OUT std_logic;
		bitDirOut : OUT std_logic
		);
	END COMPONENT;

	COMPONENT HallTradSpeedDecoder
	PORT(
		ck : IN std_logic;
		pinSA : IN std_logic;
		pinSB : IN std_logic;          
		SpeedOut : OUT std_logic_vector(12 downto 0)
		);
	END COMPONENT;

	COMPONENT segDisplay
	PORT(
		ck : IN std_logic;
		vecDfIn : IN std_logic_vector(7 downto 0);
		swDir : IN std_logic;
		SpeedIn : IN std_logic_vector(12 downto 0);
		swSelDisp : IN std_logic;          
		an : OUT std_logic_vector(3 downto 0);
		seg : OUT std_logic_vector(6 downto 0)
		);
	END COMPONENT;

  signal vecDf: STD_LOGIC_VECTOR (7 downto 0) := x"00"; -- Duty Factor for PWM
  signal Speed : STD_LOGIC_VECTOR (12 downto 0);  -- motor Speed register


begin

	Inst_adjustBcd: adjustBcd PORT MAP(
		ck => ck,
		bitCountUp => btnFaster,
		bitCountDown => btnSlower,
		vecBcdOut => vecDf
	);

	Inst_HB5RefComp: HB5RefComp PORT MAP(
		ck => ck,
		bitDirIn => swDir,
		vecDfIn => vecDf,
		bitEnOut => pinEn,
		bitDirOut => pinDir
	);

	Inst_HallTradSpeedDecoder: HallTradSpeedDecoder PORT MAP(
		ck => ck,
		pinSA => pinSA,
		pinSB => pinSB,
		SpeedOut => Speed
	);

	Inst_segDisplay: segDisplay PORT MAP(
		ck => ck,
		vecDfIn => vecDf,
		swDir => swDir,
		SpeedIn => Speed,
		swSelDisp => swSelDisp,
		an => an,
		seg => seg
	);
	
end Mixt;
